@@ -1,9 +1,6 @@
-// Copyright (C) 2010 NICTA and the authors listed below
-// http://nicta.com.au
-// 
-// Authors:
-// - Conrad Sanderson (conradsand at ieee dot org)
-// - Ian Cullinan (ian dot cullinan at nicta dot com dot au)
+// Copyright (C) 2008-2011 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2011 Conrad Sanderson
+// Copyright (C) 2009-2010 Ian Cullinan
 // 
 // This file is part of the Armadillo C++ library.
 // It is provided without any warranty of fitness
@@ -212,6 +209,29 @@ field<oT>::operator[] (const u32 i) const
   }
 
 
+
+//! linear element accessor (treats the field as a vector); no bounds check
+template<typename oT>
+arma_inline
+oT&
+field<oT>::at(const u32 i)
+  {
+  return (*mem[i]);
+  }
+  
+  
+  
+//! linear element accessor (treats the field as a vector); no bounds check
+template<typename oT>
+arma_inline
+const oT&
+field<oT>::at(const u32 i) const
+  {
+  return (*mem[i]);
+  }
+
+
+
 //! linear element accessor (treats the field as a vector); bounds checking not done when ARMA_NO_DEBUG is defined
 template<typename oT>
 arma_inline
@@ -282,6 +302,26 @@ field<oT>::at(const u32 in_row, const u32 in_col) const
 
 
 
+template<typename oT>
+inline
+field_injector< field<oT> >
+field<oT>::operator<<(const oT& val)
+  {
+  return field_injector< field<oT> >(*this, val);
+  }
+
+
+
+template<typename oT>
+inline
+field_injector< field<oT> >
+field<oT>::operator<<(const injector_end_of_row& x)
+  {
+  return field_injector< field<oT> >(*this, x);
+  }
+
+
+
 //! creation of subview_field (row of a field)
 template<typename oT>
 inline
@@ -291,7 +331,8 @@ field<oT>::row(const u32 row_num)
   arma_extra_debug_sigprint();
   
   arma_debug_check( (row_num >= n_rows), "field::row(): row out of bounds" );
-  return subview_field<oT>(*this, row_num, 0, row_num, n_cols-1);
+  
+  return subview_field<oT>(*this, row_num, 0, 1, n_cols);
   }
 
 
@@ -305,7 +346,8 @@ field<oT>::row(const u32 row_num) const
   arma_extra_debug_sigprint();
   
   arma_debug_check( (row_num >= n_rows), "field::row(): row out of bounds" );
-  return subview_field<oT>(*this, row_num, 0, row_num, n_cols-1);
+  
+  return subview_field<oT>(*this, row_num, 0, 1, n_cols);
   }
 
 
@@ -319,7 +361,8 @@ field<oT>::col(const u32 col_num)
   arma_extra_debug_sigprint();
   
   arma_debug_check( (col_num >= n_cols), "field::col(): out of bounds");
-  return subview_field<oT>(*this, 0, col_num, n_rows-1, col_num);
+  
+  return subview_field<oT>(*this, 0, col_num, n_rows, 1);
   }
 
 
@@ -333,7 +376,8 @@ field<oT>::col(const u32 col_num) const
   arma_extra_debug_sigprint();
   
   arma_debug_check( (col_num >= n_cols), "field::col(): out of bounds");
-  return subview_field<oT>(*this, 0, col_num, n_rows-1, col_num);
+  
+  return subview_field<oT>(*this, 0, col_num, n_rows, 1);
   }
 
 
@@ -352,7 +396,9 @@ field<oT>::rows(const u32 in_row1, const u32 in_row2)
     "field::rows(): indicies out of bounds or incorrectly used"
     );
   
-  return subview_field<oT>(*this, in_row1, 0, in_row2, n_cols-1);
+  const u32 sub_n_rows = in_row2 - in_row1 + 1;
+  
+  return subview_field<oT>(*this, in_row1, 0, sub_n_rows, n_cols);
   }
 
 
@@ -371,7 +417,9 @@ field<oT>::rows(const u32 in_row1, const u32 in_row2) const
     "field::rows(): indicies out of bounds or incorrectly used"
     );
   
-  return subview_field<oT>(*this, in_row1, 0, in_row2, n_cols-1);
+  const u32 sub_n_rows = in_row2 - in_row1 + 1;
+  
+  return subview_field<oT>(*this, in_row1, 0, sub_n_rows, n_cols);
   }
 
 
@@ -390,7 +438,9 @@ field<oT>::cols(const u32 in_col1, const u32 in_col2)
     "field::cols(): indicies out of bounds or incorrectly used"
     );
   
-  return subview_field<oT>(*this, 0, in_col1, n_rows-1, in_col2);
+  const u32 sub_n_cols = in_col2 - in_col1 + 1;
+  
+  return subview_field<oT>(*this, 0, in_col1, n_rows, sub_n_cols);
   }
 
 
@@ -409,7 +459,9 @@ field<oT>::cols(const u32 in_col1, const u32 in_col2) const
     "field::cols(): indicies out of bounds or incorrectly used"
     );
   
-  return subview_field<oT>(*this, 0, in_col1, n_rows-1, in_col2);
+  const u32 sub_n_cols = in_col2 - in_col1 + 1;
+  
+  return subview_field<oT>(*this, 0, in_col1, n_rows, sub_n_cols);
   }
 
 
@@ -428,12 +480,15 @@ field<oT>::subfield(const u32 in_row1, const u32 in_col1, const u32 in_row2, con
     "field::subfield(): indices out of bounds or incorrectly used"
     );
   
-  return subview_field<oT>(*this, in_row1, in_col1, in_row2, in_col2);
+  const u32 sub_n_rows = in_row2 - in_row1 + 1;
+  const u32 sub_n_cols = in_col2 - in_col1 + 1;
+  
+  return subview_field<oT>(*this, in_row1, in_col1, sub_n_rows, sub_n_cols);
   }
 
 
 
-//! creation of subview_field (generic submatrix)
+//! creation of subview_field (subfield with arbitrary dimensions)
 template<typename oT>
 inline
 const subview_field<oT>
@@ -447,7 +502,106 @@ field<oT>::subfield(const u32 in_row1, const u32 in_col1, const u32 in_row2, con
     "field::subfield(): indices out of bounds or incorrectly used"
     );
   
-  return subview_field<oT>(*this, in_row1, in_col1, in_row2, in_col2);
+  const u32 sub_n_rows = in_row2 - in_row1 + 1;
+  const u32 sub_n_cols = in_col2 - in_col1 + 1;
+  
+  return subview_field<oT>(*this, in_row1, in_col1, sub_n_rows, sub_n_cols);
+  }
+
+
+
+//! creation of subview_field (subfield with arbitrary dimensions)
+template<typename oT>
+inline
+subview_field<oT>
+field<oT>::subfield(const span& row_span, const span& col_span)
+  {
+  arma_extra_debug_sigprint();
+  
+  const bool row_all = row_span.whole;
+  const bool col_all = col_span.whole;
+  
+  const u32 local_n_rows = n_rows;
+  const u32 local_n_cols = n_cols;
+  
+  const u32 in_row1    = row_all ? 0            : row_span.a;
+  const u32 in_row2    =                          row_span.b;
+  const u32 sub_n_rows = row_all ? local_n_rows : in_row2 - in_row1 + 1;
+  
+  const u32 in_col1    = col_all ? 0            : col_span.a;
+  const u32 in_col2    =                          col_span.b;
+  const u32 sub_n_cols = col_all ? local_n_cols : in_col2 - in_col1 + 1;
+  
+  arma_debug_check
+    (
+    ( row_all ? false : ((in_row1 > in_row2) || (in_row2 >= local_n_rows)) )
+    ||
+    ( col_all ? false : ((in_col1 > in_col2) || (in_col2 >= local_n_cols)) )
+    ,
+    "field::subfield(): indices out of bounds or incorrectly used"
+    );
+  
+  return subview_field<oT>(*this, in_row1, in_col1, sub_n_rows, sub_n_cols);
+  }
+
+
+
+//! creation of subview_field (subfield with arbitrary dimensions)
+template<typename oT>
+inline
+const subview_field<oT>
+field<oT>::subfield(const span& row_span, const span& col_span) const
+  {
+  arma_extra_debug_sigprint();
+  
+  const bool row_all = row_span.whole;
+  const bool col_all = col_span.whole;
+  
+  const u32 local_n_rows = n_rows;
+  const u32 local_n_cols = n_cols;
+  
+  const u32 in_row1    = row_all ? 0            : row_span.a;
+  const u32 in_row2    =                          row_span.b;
+  const u32 sub_n_rows = row_all ? local_n_rows : in_row2 - in_row1 + 1;
+  
+  const u32 in_col1    = col_all ? 0            : col_span.a;
+  const u32 in_col2    =                          col_span.b;
+  const u32 sub_n_cols = col_all ? local_n_cols : in_col2 - in_col1 + 1;
+  
+  arma_debug_check
+    (
+    ( row_all ? false : ((in_row1 > in_row2) || (in_row2 >= local_n_rows)) )
+    ||
+    ( col_all ? false : ((in_col1 > in_col2) || (in_col2 >= local_n_cols)) )
+    ,
+    "field::subfield(): indices out of bounds or incorrectly used"
+    );
+  
+  return subview_field<oT>(*this, in_row1, in_col1, sub_n_rows, sub_n_cols);
+  }
+
+
+
+template<typename oT>
+inline
+subview_field<oT>
+field<oT>::operator()(const span& row_span, const span& col_span)
+  {
+  arma_extra_debug_sigprint();
+  
+  return (*this).subfield(row_span, col_span);
+  }
+
+
+
+template<typename oT>
+inline
+const subview_field<oT>
+field<oT>::operator()(const span& row_span, const span& col_span) const
+  {
+  arma_extra_debug_sigprint();
+  
+  return (*this).subfield(row_span, col_span);
   }
 
 
@@ -530,6 +684,7 @@ field<oT>::fill(const oT& x)
 
 
 
+//! reset the field to an empty state (i.e. the field will have no objects)
 template<typename oT>
 inline
 void
@@ -542,6 +697,7 @@ field<oT>::reset()
 
 
 
+//! reset each object
 template<typename oT>
 inline
 void
@@ -554,6 +710,133 @@ field<oT>::reset_objects()
 
 
 
+//! returns true if the field has no objects
+template<typename oT>
+arma_inline
+bool
+field<oT>::is_empty() const
+  {
+  return (n_elem == 0);
+  }
+
+
+
+//! returns true if the given index is currently in range
+template<typename oT>
+arma_inline
+arma_warn_unused
+bool
+field<oT>::in_range(const u32 i) const
+  {
+  return (i < n_elem);
+  }
+
+
+
+//! returns true if the given start and end indices are currently in range
+template<typename oT>
+arma_inline
+arma_warn_unused
+bool
+field<oT>::in_range(const span& x) const
+  {
+  arma_extra_debug_sigprint();
+  
+  if(x.whole == true)
+    {
+    return true;
+    }
+  else
+    {
+    const u32 a = x.a;
+    const u32 b = x.b;
+    
+    return ( (a <= b) && (b < n_elem) );
+    }
+  }
+
+
+
+//! returns true if the given location is currently in range
+template<typename oT>
+arma_inline
+arma_warn_unused
+bool
+field<oT>::in_range(const u32 in_row, const u32 in_col) const
+  {
+  return ( (in_row < n_rows) && (in_col < n_cols) );
+  }
+
+
+
+template<typename oT>
+arma_inline
+arma_warn_unused
+bool
+field<oT>::in_range(const span& row_span, const u32 in_col) const
+  {
+  arma_extra_debug_sigprint();
+  
+  if(row_span.whole == true)
+    {
+    return (in_col < n_cols);
+    }
+  else
+    {
+    const u32 in_row1 = row_span.a;
+    const u32 in_row2 = row_span.b;
+    
+    return ( (in_row1 <= in_row2) && (in_row2 < n_rows) && (in_col < n_cols) );
+    }
+  }
+
+
+
+template<typename oT>
+arma_inline
+arma_warn_unused
+bool
+field<oT>::in_range(const u32 in_row, const span& col_span) const
+  {
+  arma_extra_debug_sigprint();
+  
+  if(col_span.whole == true)
+    {
+    return (in_row < n_rows);
+    }
+  else
+    {
+    const u32 in_col1 = col_span.a;
+    const u32 in_col2 = col_span.b;
+  
+    return ( (in_row < n_rows) && (in_col1 <= in_col2) && (in_col2 < n_cols) );
+    }
+  }
+
+
+
+template<typename oT>
+arma_inline
+arma_warn_unused
+bool
+field<oT>::in_range(const span& row_span, const span& col_span) const
+  {
+  arma_extra_debug_sigprint();
+  
+  const u32 in_row1 = row_span.a;
+  const u32 in_row2 = row_span.b;
+  
+  const u32 in_col1 = col_span.a;
+  const u32 in_col2 = col_span.b;
+  
+  const bool rows_ok = row_span.whole ? true : ( (in_row1 <= in_row2) && (in_row2 < n_rows) );
+  const bool cols_ok = col_span.whole ? true : ( (in_col1 <= in_col2) && (in_col2 < n_cols) );
+  
+  return ( (rows_ok == true) && (cols_ok == true) );
+  }
+
+
+
 template<typename oT>
 inline
 bool
@@ -568,11 +851,11 @@ field<oT>::save(const std::string name, const file_type type, const bool print_s
     {
     if(err_msg.length() > 0)
       {
-      arma_print("field::save(): ", err_msg, name);
+      arma_warn(true, "field::save(): ", err_msg, name);
       }
     else
       {
-      arma_print("field::save(): couldn't write to ", name);
+      arma_warn(true, "field::save(): couldn't write to ", name);
       }
     }
   
@@ -595,11 +878,11 @@ field<oT>::save(std::ostream& os, const file_type type, const bool print_status)
     {
     if(err_msg.length() > 0)
       {
-      arma_print("field::save(): ", err_msg, "[ostream]");
+      arma_warn(true, "field::save(): ", err_msg, "[ostream]");
       }
     else
       {
-      arma_print("field::save(): couldn't write to [ostream]");
+      arma_warn(true, "field::save(): couldn't write to [ostream]");
       }
     }
   
@@ -622,11 +905,11 @@ field<oT>::load(const std::string name, const file_type type, const bool print_s
     {
     if(err_msg.length() > 0)
       {
-      arma_print("field::load(): ", err_msg, name);
+      arma_warn(true, "field::load(): ", err_msg, name);
       }
     else
       {
-      arma_print("field::load(): couldn't read from ", name);
+      arma_warn(true, "field::load(): couldn't read from ", name);
       }
     }
   
@@ -654,11 +937,11 @@ field<oT>::load(std::istream& is, const file_type type, const bool print_status)
     {
     if(err_msg.length() > 0)
       {
-      arma_print("field::load(): ", err_msg, "[istream]");
+      arma_warn(true, "field::load(): ", err_msg, "[istream]");
       }
     else
       {
-      arma_print("field::load(): couldn't read from [istream]");
+      arma_warn(true, "field::load(): couldn't read from [istream]");
       }
     }
   
@@ -778,7 +1061,7 @@ field<oT>::init(const u32 n_rows_in, const u32 n_cols_in)
     else
       {
       mem = new(std::nothrow) oT* [n_elem_new];
-      arma_check( (mem == 0), "field::init(): out of memory" );
+      arma_check_bad_alloc( (mem == 0), "field::init(): out of memory" );
       }
     
     access::rw(n_elem) = n_elem_new;
@@ -1666,6 +1949,8 @@ field_aux::save(const field< std::string >& x, const std::string& name, const fi
   {
   arma_extra_debug_sigprint();
   
+  arma_ignore(type);
+  
   err_msg.clear();
   
   return diskio::save_std_string(x, name);
@@ -1679,6 +1964,8 @@ field_aux::save(const field< std::string >& x, std::ostream& os, const file_type
   {
   arma_extra_debug_sigprint();
   
+  arma_ignore(type);
+  
   err_msg.clear();
   
   return diskio::save_std_string(x, os);
@@ -1692,6 +1979,8 @@ field_aux::load(field< std::string >& x, const std::string& name, const file_typ
   {
   arma_extra_debug_sigprint();
   
+  arma_ignore(type);
+  
   return diskio::load_std_string(x, name, err_msg);
   }
 
@@ -1703,9 +1992,17 @@ field_aux::load(field< std::string >& x, std::istream& is, const file_type type,
   {
   arma_extra_debug_sigprint();
   
+  arma_ignore(type);
+  
   return diskio::load_std_string(x, is, err_msg);
   }
 
 
 
+#ifdef ARMA_EXTRA_FIELD_MEAT
+  #include ARMA_INCFILE_WRAP(ARMA_EXTRA_FIELD_MEAT)
+#endif
+
+
+
 //! @}
