@@ -1,7 +1,5 @@
-// Copyright (C) 2009 NICTA
-// 
-// Authors:
-// - Conrad Sanderson (conradsand at ieee dot org)
+// Copyright (C) 2008-2010 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2010 Conrad Sanderson
 // 
 // This file is part of the Armadillo C++ library.
 // It is provided without any warranty of fitness
@@ -21,7 +19,7 @@
 namespace arma_boost
   {
   
-  #if defined(ARMA_USE_BOOST)
+  #if defined(ARMA_USE_BOOST_FORMAT)
 
     using boost::format;
     using boost::basic_format;
@@ -145,7 +143,7 @@ namespace arma_boost
           buffer = new char[buffer_size];
           }
         
-        required_size = arma_snprintf(buffer, buffer_size, X.A.A.c_str(), X.B);
+        required_size = arma_snprintf(buffer, size_t(buffer_size), X.A.A.c_str(), X.B);
         
         if(required_size < buffer_size)
           {
@@ -197,7 +195,7 @@ namespace arma_boost
           buffer = new char[buffer_size];
           }
         
-        required_size = arma_snprintf(buffer, buffer_size, X.A.A.A.c_str(), X.A.B, X.B);
+        required_size = arma_snprintf(buffer, size_t(buffer_size), X.A.A.A.c_str(), X.A.B, X.B);
         
         if(required_size < buffer_size)
           {
@@ -249,7 +247,7 @@ namespace arma_boost
           buffer = new char[buffer_size];
           }
         
-        required_size = arma_snprintf(buffer, buffer_size, X.A.A.A.A.c_str(), X.A.A.B, X.A.B, X.B);
+        required_size = arma_snprintf(buffer, size_t(buffer_size), X.A.A.A.A.c_str(), X.A.A.B, X.A.B, X.B);
         
         if(required_size < buffer_size)
           {
@@ -301,7 +299,7 @@ namespace arma_boost
           buffer = new char[buffer_size];
           }
         
-        required_size = arma_snprintf(buffer, buffer_size, X.A.A.A.A.A.c_str(), X.A.A.A.B, X.A.A.B, X.A.B, X.B);
+        required_size = arma_snprintf(buffer, size_t(buffer_size), X.A.A.A.A.A.c_str(), X.A.A.A.B, X.A.A.B, X.A.B, X.B);
         
         if(required_size < buffer_size)
           {
@@ -353,7 +351,7 @@ namespace arma_boost
           buffer = new char[buffer_size];
           }
         
-        required_size = arma_snprintf(buffer, buffer_size, X.A.A.A.A.A.A.c_str(), X.A.A.A.A.B, X.A.A.A.B, X.A.A.B, X.A.B, X.B);
+        required_size = arma_snprintf(buffer, size_t(buffer_size), X.A.A.A.A.A.A.c_str(), X.A.A.A.A.B, X.A.A.A.B, X.A.A.B, X.A.B, X.B);
         
         if(required_size < buffer_size)
           {
@@ -405,7 +403,7 @@ namespace arma_boost
           buffer = new char[buffer_size];
           }
         
-        required_size = arma_snprintf(buffer, buffer_size, X.A.A.A.A.A.A.A.c_str(), X.A.A.A.A.A.B, X.A.A.A.A.B, X.A.A.A.B, X.A.A.B, X.A.B, X.B);
+        required_size = arma_snprintf(buffer, size_t(buffer_size), X.A.A.A.A.A.A.A.c_str(), X.A.A.A.A.A.B, X.A.A.A.A.B, X.A.A.A.B, X.A.A.B, X.A.B, X.B);
         
         if(required_size < buffer_size)
           {
@@ -491,6 +489,63 @@ namespace arma_boost
         
   #endif
   
+  
+  template<typename T> struct string_only              { };
+  template<>           struct string_only<std::string> { typedef std::string result; };
+  
+  template<typename T> struct char_only                { };
+  template<>           struct char_only<char         > { typedef char        result; };
+
+  template<typename T>
+  struct basic_format_only { };
+  
+  #if defined(ARMA_USE_BOOST_FORMAT)
+    template<typename T>
+    struct basic_format_only< basic_format<T>      > { typedef basic_format<T>     result; };
+  #else
+    template<typename T1, typename T2>
+    struct basic_format_only< basic_format<T1, T2> > { typedef basic_format<T1,T2> result; };
+  #endif
+
+
+
+  template<typename T1>
+  inline
+  static
+  const T1&
+  str_wrapper(const T1& x, const typename string_only<T1>::result* junk = 0)
+    {
+    arma_ignore(junk);
+    
+    return x;
+    }
+  
+  
+  
+  template<typename T1>
+  inline
+  static
+  const T1*
+  str_wrapper(const T1* x, const typename char_only<T1>::result* junk = 0)
+    {
+    arma_ignore(junk);
+    
+    return x;
+    }
+  
+  
+  
+  template<typename T1>
+  inline
+  static
+  std::string
+  str_wrapper(const T1& x, const typename basic_format_only<T1>::result* junk = 0)
+    {
+    arma_ignore(junk);
+    
+    return str(x);
+    }
+  
   }
 
 //! @}
