@@ -1,8 +1,5 @@
-// Copyright (C) 2010 NICTA and the authors listed below
-// http://nicta.com.au
-// 
-// Authors:
-// - Conrad Sanderson (conradsand at ieee dot org)
+// Copyright (C) 2009-2011 NICTA (www.nicta.com.au)
+// Copyright (C) 2009-2011 Conrad Sanderson
 // 
 // This file is part of the Armadillo C++ library.
 // It is provided without any warranty of fitness
@@ -90,7 +87,16 @@ running_stat_vec<eT>::operator() (const Base<typename get_pod_type<eT>::result,
   const unwrap<T1>        tmp(X.get_ref());
   const Mat<eT>& sample = tmp.M;
   
-  arma_check( (sample.is_finite() == false), "running_stat_vec: given sample has non-finite elements" );
+  if( sample.is_empty() )
+    {
+    return;
+    }
+  
+  if( sample.is_finite() == false )
+    {
+    arma_warn(true, "running_stat_vec: sample ignored as it has non-finite elements");
+    return;
+    }
   
   running_stat_vec_aux::update_stats(*this, sample);
   }
@@ -112,7 +118,16 @@ running_stat_vec<eT>::operator() (const Base<std::complex<typename get_pod_type<
   const unwrap<T1>        tmp(X.get_ref());
   const Mat<eT>& sample = tmp.M;
   
-  arma_check( (sample.is_finite() == false), "running_stat_vec: given sample has non-finite elements" );
+  if( sample.is_empty() )
+    {
+    return;
+    }
+  
+  if( sample.is_finite() == false )
+    {
+    arma_warn(true, "running_stat_vec: sample ignored as it has non-finite elements");
+    return;
+    }
   
   running_stat_vec_aux::update_stats(*this, sample);
   }
@@ -280,7 +295,7 @@ const Mat<eT>&
 running_stat_vec<eT>::min() const
   {
   arma_extra_debug_sigprint();
-
+  
   return min_val;
   }
 
@@ -293,12 +308,25 @@ const Mat<eT>&
 running_stat_vec<eT>::max() const
   {
   arma_extra_debug_sigprint();
-
+  
   return max_val;
   }
 
 
 
+//! number of samples so far
+template<typename eT>
+inline
+typename get_pod_type<eT>::result
+running_stat_vec<eT>::count() const
+  {
+  arma_extra_debug_sigprint();
+  
+  return counter.value();
+  }
+
+
+
 //
 
 
@@ -463,11 +491,11 @@ running_stat_vec_aux::update_stats(running_stat_vec< std::complex<T> >& x, const
       
       if(sample.n_cols == 1)
         {
-        tmp2 = conj(tmp1)*trans(tmp1);
+        tmp2 = conj(tmp1)*strans(tmp1);
         }
       else
         {
-        tmp2 = trans(conj(tmp1))*tmp1;
+        tmp2 = trans(tmp1)*tmp1;  //tmp2 = strans(conj(tmp1))*tmp1;
         }
       
       x.r_cov *= (N_minus_1/N);
