@@ -11,41 +11,41 @@ namespace arma
   {
   namespace junk
     {
-
+    
     #if defined(ARMA_USE_ATLAS)
       void
       pull_atlas()
         {
-        int    x;
-        double y;
-
+        int    x = int(0);
+        double y = double(0);
+        
         arma::atlas::clapack_dgetrf(CblasColMajor, x, x, &y, x, &x);
         arma::atlas::cblas_dgemm(CblasColMajor, CblasNoTrans, CblasNoTrans, x, x, x, y, &y, x, &y, x, y, &y, x);
         }
     #endif
-
+    
     #if defined(ARMA_USE_LAPACK)
       void
       pull_lapack()
         {
-        int    x;
-        double y;
-
-        arma::lapack::dgetrf_(&x, &x, &y, &x, &x, &x);
+        blas_int x = blas_int(0);
+        double   y = double(0);
+        
+        arma::lapack::getrf(&x, &x, &y, &x, &x, &x);
         }
     #endif
-
+    
     #if defined(ARMA_USE_BLAS)
       void
       pull_blas()
         {
-        char   c;
-        int    x;
-        double y;
-
-        arma::blas::dgemm_(&c, &c, &x, &x, &x, &y, &y, &x, &y, &x, &y, &y, &x);
+        char     c = char(0);
+        blas_int x = blas_int(0);
+        double   y = double(0);
+        
+        arma::blas::gemm(&c, &c, &x, &x, &x, &y, &y, &x, &y, &x, &y, &y, &x);
         }
     #endif
-
+    
     }
   }
